/**************************************************************************\
*   BC 1.01, WC 10.0                            Source: pgped.c
*   Host platform:   DOS                        Code:   ASCII-7
*   Target platform: DOS                        Code:   ASCII-7
*   Project name:    PGPEd: shell for edit pgp'ed files
*   Version:         0.2.0                      Started at:   07.11.98
*   Last change:     14.11.98                   Elapsed time: 0026 hours
*   Programmer:      Andrew Porokhnyak
****************************************************************************
*
**** 0.2.0  14.11.1998
*
**** 0.1.1  12.11.1998
*
**** 0.1.0  11.11.1998  First working release
*
\**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "defines.h"
#ifdef HAVE_SPAWN
#include <process.h>
#endif


#include "config.h"
#include "initvars.h"
#include "tools.h"
#include "version.h"

void usage(void);

int main(int argc, char **argv) {

    char passw[MAXPASSW];
    char cmd[MAXPATH];
    int rmode; /* programm run mode */
    int retc;


    /* I. Read config, parse command line, initialize variables */
    if (strchr(CONFIG, PATH_DELIM[0])) {
        /* config name with path -- assume absolute location */
        readcfg(CONFIG);
    } else {
        /* config name without path -- find it in the pgped bin directory */
        char *p, scfg[MAXPATH];
        strncpy(scfg, argv[0], MAXPATH);
        p=strrchr(scfg, PATH_DELIM[0]);
        if (p) {
            *(p+1)=0;
            strcat(scfg, CONFIG);
        } else {
            strncpy(scfg, CONFIG, MAXPATH);
        }
        readcfg(scfg);
    }

    rmode=parsecl(argc, argv);
    switch (rmode) {
        case RUN_HELP:
        usage();
        return 1;

        case RUN_VERSION:
        printf("%s %s /%s\n", PROD_NAME, PROD_VERS, OS);
        return 1;
    }
    initvars();

    /* II. Ask password for edited file, store it to $pass */
    if (!askpassw(ct_new, ctname, passw)) return 2;
#ifdef UNIQUE_PASSW
    strlower(fname);
    strcat(passw, fname);
#endif

    /* III. Decrypt ciphertext (if any) to plaintext "pgp -z $pass $ct" */
    if (!ct_new) {
#ifdef HAVE_SPAWN
        retc=spawnlpe(P_WAIT, pgpbin, pgpbin, "+batchmode", "-z", passw,
        ctname, NULL, envl);
#else
        sprintf(cmd, "%s +batchmode -z %s %s", pgpbin, passw, ctname);
        retc=system(cmd);
#endif
        if (retc!=pgprc_decr) {
            Log(2, "Error while decrypting ciphertext, rc=%i", retc);
            fwipe(ptname);
            return 1;
        }
    }

    /* IV. Spawn to user-selected editor for editing plaintext */
    if (rmode==RUN_EDIT) {
#ifdef HAVE_SPAWN
        retc=spawnlp(P_WAIT, editbin, editbin, ptname, NULL);
#else
        sprintf(cmd, "%s %s", editbin, ptname);
        retc=system(cmd);
#endif
    } else {
#ifdef HAVE_SPAWN
        retc=spawnlp(P_WAIT, viewbin, viewbin, ptname, NULL);
#else
        sprintf(cmd, "%s %s", viewbin, ptname);
        retc=system(cmd);
#endif
    }

    /* V. If plaintext was changed, then crypt it "pgp -c -z $pass $pt"*/
    if (rmode==RUN_EDIT) {
        rename(ctname, svname);
#ifdef HAVE_SPAWN
        retc=spawnlpe(P_WAIT, pgpbin, pgpbin, "+batchmode", "-ct", "-z",
        passw, "-o", ctname, ptname, NULL, envl);
#else
        sprintf(cmd, "%s +batchmode -ct -z %s -o %s %s", pgpbin, passw,
        ctname, ptname);
        retc=system(cmd);
#endif
        if (retc!=pgprc_encr) {
            rename(svname, ctname);
            Log(2, "Error while encrypting plaintext, rc=%i", retc);
        } else {
            fdelete(svname);
        }
    }

    /* VI. Wipe plaintext file */
    fwipe(ptname);

    return 0;
} /*----------------------------------------------------------------------*/


/*
 * usage()
 */
void usage(void) {

    printf(
"%s v%s /%s\n"
"Usage:\n"
"\tpgped [options] ciphertext\n\n"
"ciphertext - name of the file with encrypted text, if no extention given,\n"
"\t'%s' assumed\n"
"Options:\n"
"\t-v, --view - view-only mode\n"
"\t--version - version info\n"
"\t-?, -h, --help - show this help\n"

    , PROD_NAME, PROD_VERS, OS, CT_EXT);
}

/*** EOF pgped.c **********************************************************/