/**************************************************************************\
*   BC 1.0, WC 10.0, GCC 2.7                    Source: tools.c
*   Host platform:   DOS                        Code:   ASCII-7
*   Target platform: DOS                        Code:   ASCII-7
*   Project name:    PGPEd
*   Last change:     14.11.98
*   Programmer:      Andrew Porokhnyak
****************************************************************************
*
\**************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#if defined(__TURBOC__) || defined(__BORLANDC__) || defined(__WATCOMC__)
#include <io.h>
#endif

#include "defines.h"

#ifdef HAVE_CURSES
#include <curses.h>
#else
#include <conio.h>
#endif

#include "config.h"
#include "tools.h"
#include "initvars.h"


/*
 * Lowercase the string
 */
char *strlower (char *s) {
    int i;

    for (i=0; s[i]; ++i)
        s[i]=tolower(s[i]);
    return s;
} /*----------------------------------------------------------------------*/


/*
 * Log various messages to console if it level is lower then conlog
 */
void Log(int lev, char *s,...) {

    char stmp[256];
    va_list ap;
    int conlog=9;

    /* chars to ident loglevel  0123456789 */
    static const char *marks = "!?+- *$#@.";
    char ch = (0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ';
    /* Loglevels are:
    *   0 (!) Fatal error, program aborted
    *   1 (?) Non-fatal error, program will continued
    *   2 (+) Most valuable info about proram run
    *   3 (-) Less valuable info about proram run
    *   4 ( ) Normal info
    *   5 (*) Redundance info
    *   6 ($) Important variables values
    *   7 (#) Small debug
    *   8 (@) Medium debug
    *   9 (.) Full debug
    */

    va_start(ap, s);
    vsprintf(stmp, s, ap);
    va_end (ap);

    if (lev <= conlog) {
        printf("%c %s\n", ch, stmp);
    }

    if (lev==0)
        exit(1);

} /*----------------------------------------------------------------------*/


/*
 * Get password string from console (without echo)
 * Use conio or curses lib
 */
void getpw(char *pw, int maxl) {
    int c;
    int i=0;

#ifdef HAVE_CURSES
    /* std curses init sequence */
    initscr(); cbreak(); noecho();
#endif
    pw[0]=(char) 0;
    do {
        c=getch();
        switch (c) {
            case 8: if (i>0) i--; break;
	    case '\n': break;
	    case '\r': break;
            default: pw[i]=(char) c; i++; break;
        }
    } while ((i<maxl-1) && (c!=13) && (c!=10));
    pw[i]=(char) 0;
#ifdef HAVE_CURSES
    /* resume normal tty mode */
    endwin();
#endif
} /*----------------------------------------------------------------------*/


/*
 * Ask a password (twice for a new file)
 */
int askpassw(const char fornew, const char *ctname, char *passw) {
    char passr[MAXPASSW];

    fprintf(stderr, "Enter password for %s%s (empty passw will exit)\n"
        "Password: ",
        (fornew ? "new file " : ""), ctname);
    getpw(passw, MAXPASSW);
    fprintf(stderr, "\n");
    if (!passw[0]) return 0;

    if (fornew) {
        fprintf(stderr, "Repeat: ");
        getpw(passr, MAXPASSW);
        fprintf(stderr, "\n");
        if (strcmp(passw, passr)) {
            fprintf(stderr, "Passwords don't match, exit\n");
            return 0;
        }
    }

    return 1;
} /*----------------------------------------------------------------------*/

/*
 * Delete a file
 */
int fdelete(const char *path) {
    int rc;

    if ((rc = unlink (path)) == -1)
        Log(1, "error deleting `%s': %s", path, strerror (errno));
    else
        Log(9, "deleted `%s'", path);

    return rc;
} /*----------------------------------------------------------------------*/


/*
 * Wipe a file, then delete it
 */
int fwipe(const char *path) {
    int h;
    int i;
    long l, blck;
    char buff[512];

    if ((h = open(path, O_RDWR)) == -1) {
        Log(1, "cannot wipe `%s': %s", path, strerror (errno));
        return -1;
    } else {
    /*    long int flen=filelength(h); */
        long int flen=lseek(h, 0L, SEEK_END);
        if (flen<0) flen=2048L;
        blck=flen / sizeof(buff) + 1;

        lseek(h, 0L, SEEK_SET);
        for (i=0; i<sizeof(buff); i++) buff[i]=rand() % 256;
        for (l=0; l<blck; l++) write(h, buff, sizeof(buff));

        Log(9, "wiped `%s'", path);
        close(h);

        fdelete(path);
        return 0;
    }
} /*----------------------------------------------------------------------*/


/*** EOF tools.c **********************************************************/